@@ -1,17 +1,18 @@
-.form_results <- function(trait, selected_loci,   fformula, indxNA_pheno, indxNA_geno,
-                           ncpu, availmemGb, quiet,  extBIC, gamma, geno, pheno, map, Zmat, outlierstat )
+.form_results <- function(traitname, trait, selected_loci,   fformula, indxNA_pheno, 
+                           ncpu, ngpu, availmemGb, quiet,  extBIC, gamma, geno, pheno, map, Zmat, outlierstat )
 {
   ## internal function - used by AM for forming the results object
   if (length(selected_loci) > 1){
-   sigres <- list(trait=trait,
+   sigres <- list(traitname=traitname, 
+                    trait=trait,
                     fformula = fformula,
                     indxNA_pheno = indxNA_pheno,
-                    indxNA_geno = indxNA_geno,
                     Mrk=map[[1]][selected_loci],
                     Chr=map[[2]][selected_loci],
                     Pos=map[[3]][selected_loci],
                     Indx=selected_loci,
                     ncpu=ncpu,
+                    ngpu=ngpu,
                     availmemGb=availmemGb,
                     quiet=quiet,
                     extBIC=extBIC,
@@ -20,17 +21,18 @@
                     pheno=pheno,
                     map=map,
                     Zmat=Zmat,
-                    outlierstat=outlierstat)
+                    outlierstat=outlierstat) 
   } else {
-   sigres <- list(trait=trait,
+   sigres <- list(traitname=traitname,
+                    trait=trait,
                     fformula = fformula,
                     indxNA_pheno = indxNA_pheno,
-                    indxNA_geno = indxNA_geno,
                     Mrk=NA,
                     Chr=NA,
                     Pos=NA,
                     Indx=selected_loci,
                     ncpu=ncpu,
+                    ngpu=ngpu,
                     availmemGb=availmemGb,
                     quiet=quiet,
                     extBIC=extBIC,
@@ -39,9 +41,11 @@
                     pheno=pheno,
                     map=map,
                     Zmat=Zmat, 
-                    outlierstat=outlierstat)
-  }
+                    outlierstat=outlierstat) 
+}
+
 return(sigres)
 }
 
 
+
