@@ -4,16 +4,21 @@
   if (length(selected_loci) == 1 & any(is.na(selected_loci)))
   {
       message("No significant marker-trait associations have been found. \n\n")
+     if (is.null(gamma)){
+        message("Gamma value for model selection was set to its default value of 1. ")
+     } else {
+        message("Gamma value for model selection was set to  ", round(gamma,3) , " ")
+     } 
+
   }  else {
      .print_results(selected_loci=selected_loci, map=map,  extBIC=extBIC)
      if (is.null(gamma)){
-        message("Gamma value for model selection was set to its default value of 1. \n")
+        message("Gamma value for model selection was set to its default value of 1. ")
      } else {
-        message("Gamma value for model selection was set to  ", round(gamma,3) , " \n")
+        message("Gamma value for model selection was set to  ", round(gamma,3) , " ")
      } 
-     message("\n\n")
   }   ## end if else
-
+  message("")
 
 }  ## end function print.finals
 
